@@ -75,7 +75,7 @@ void show_notification(const gchar *summary, const gchar *message,
 		notify_notification_close(notify, NULL);
 	}
 
-	notify = notify_notification_new(summary, message, ACTIVE_ICON_NAME, NULL);
+	notify = notify_notification_new(summary, message, ACTIVE_ICON_NAME);
 
 	notify_notification_set_timeout(notify, timeout);
 
@@ -122,6 +122,10 @@ GtkStatusIcon *init_notification(void)
 				   _("Bluetooth"));
 	gtk_status_icon_set_tooltip_markup(statusicon, tooltip);
 
+	/* XXX: Make sure the status icon is actually shown */
+	gtk_status_icon_set_visible(statusicon, FALSE);
+	gtk_status_icon_set_visible(statusicon, TRUE);
+
 	return statusicon;
 }
 
@@ -164,18 +168,3 @@ void set_icon(gboolean enabled)
 		gtk_status_icon_set_tooltip_markup(statusicon, _tooltip);
 	}
 }
-
-void enable_blinking(void)
-{
-	gtk_status_icon_set_blinking(statusicon, TRUE);
-}
-
-void disable_blinking(void)
-{
-	gtk_status_icon_set_blinking(statusicon, FALSE);
-}
-
-gboolean query_blinking(void)
-{
-	return gtk_status_icon_get_blinking(statusicon);
-}
