@@ -167,9 +167,13 @@ filter_type_changed_cb (GtkComboBox *widget, gpointer data)
 void
 bluetooth_filter_widget_set_title (BluetoothFilterWidget *self, gchar *title)
 {
+	gchar *markup;
 	BluetoothFilterWidgetPrivate *priv = BLUETOOTH_FILTER_WIDGET_GET_PRIVATE(self);
 
-	gtk_label_set_text (GTK_LABEL (priv->title), title);
+	markup = g_strdup_printf ("<b>%s</b>", title);
+	gtk_label_set_text (GTK_LABEL (priv->title), markup);
+	g_free (markup);
+
 	gtk_label_set_use_markup (GTK_LABEL (priv->title), TRUE);
 }
 
@@ -224,7 +228,7 @@ bluetooth_filter_widget_init(BluetoothFilterWidget *self)
 	gtk_box_set_spacing (GTK_BOX (self), 6);
 
 	priv->title = gtk_label_new ("");
-	bluetooth_filter_widget_set_title (self, _("<b>Show Only Bluetooth Devices With...</b>"));
+	bluetooth_filter_widget_set_title (self, _("Show Only Bluetooth Devices With..."));
 	gtk_widget_show (priv->title);
 	gtk_box_pack_start (GTK_BOX (self), priv->title, TRUE, TRUE, 0);
 	gtk_misc_set_alignment (GTK_MISC (priv->title), 0, 0.5);
@@ -249,7 +253,7 @@ bluetooth_filter_widget_init(BluetoothFilterWidget *self)
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	priv->device_category_label = label;
 
-	priv->device_category = gtk_combo_box_new_text ();
+	priv->device_category = gtk_combo_box_text_new ();
 	gtk_widget_set_no_show_all (priv->device_category, TRUE);
 	gtk_widget_show (priv->device_category);
 	gtk_table_attach (GTK_TABLE (table), priv->device_category, 1, 2, 1, 2,
@@ -257,8 +261,8 @@ bluetooth_filter_widget_init(BluetoothFilterWidget *self)
 			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
 	gtk_widget_set_tooltip_text (priv->device_category, _("Select the device category to filter"));
 	for (i = 0; i < BLUETOOTH_CATEGORY_NUM_CATEGORIES; i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (priv->device_category),
-					   _(bluetooth_device_category_to_string (i)));
+		gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (priv->device_category),
+					        _(bluetooth_device_category_to_string (i)));
 	}
 	g_signal_connect (G_OBJECT (priv->device_category), "changed",
 			  G_CALLBACK (filter_category_changed_cb), self);
