@@ -81,6 +81,8 @@ static char *pincode = NULL;
 static GtkBuilder *builder = NULL;
 
 static GtkAssistant *window_assistant = NULL;
+static GtkWidget *button_close = NULL;
+static GtkWidget *button_cancel = NULL;
 static GtkWidget *page_search = NULL;
 static GtkWidget *page_connecting = NULL;
 static GtkWidget *page_setup = NULL;
@@ -115,6 +117,7 @@ static GtkWidget *radio_auto = NULL;
 static GtkWidget *radio_0000 = NULL;
 static GtkWidget *radio_1111 = NULL;
 static GtkWidget *radio_1234 = NULL;
+static GtkWidget *radio_none = NULL;
 static GtkWidget *radio_custom = NULL;
 static GtkWidget *entry_custom = NULL;
 
@@ -349,7 +352,7 @@ connect_callback (BluetoothClient *_client,
 	}
 
 	if (success == FALSE)
-		g_message ("Failed to connect to device %s", data->path);
+		g_debug ("Failed to connect to device %s", data->path);
 
 	g_timer_destroy (data->timer);
 	g_free (data->path);
@@ -409,88 +412,15 @@ close_callback (GtkWidget *assistant,
 	gtk_main_quit();
 }
 
-/* HACK, to access the GtkAssistant buttons */
-struct RealGtkAssistant
-{
-	GtkWindow  parent;
-
-	GtkWidget *cancel;
-	GtkWidget *forward;
-	GtkWidget *back;
-	GtkWidget *apply;
-	GtkWidget *close;
-	GtkWidget *last;
-
-	/*< private >*/
-	GtkAssistantPrivate *priv;
-};
-typedef struct RealGtkAssistant RealGtkAssistant;
-
-static gboolean
-prepare_idle_cb (gpointer data)
-{
-	RealGtkAssistant *assistant = (RealGtkAssistant *) window_assistant;
-	gint page;
-
-	page = gtk_assistant_get_current_page (GTK_ASSISTANT (window_assistant));
-	if (page == PAGE_FAILURE) {
-		gtk_widget_hide (assistant->cancel);
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_show (assistant->close);
-		gtk_widget_set_sensitive (assistant->close, TRUE);
-	}
-	if (page == PAGE_CONNECTING) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		gtk_widget_show (assistant->cancel);
-		gtk_widget_set_sensitive (assistant->cancel, TRUE);
-	}
-	if (page == PAGE_SETUP) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		gtk_widget_show (assistant->cancel);
-		gtk_widget_set_sensitive (assistant->cancel, TRUE);
-	}
-	if (page == PAGE_SSP_SETUP) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->close);
-		if (display_called == FALSE) {
-			gtk_widget_hide (assistant->cancel);
-		} else {
-			gtk_widget_show (assistant->cancel);
-			gtk_widget_set_sensitive (assistant->cancel, TRUE);
-		}
-	}
-	if (page == PAGE_FINISHING) {
-		gtk_widget_hide (assistant->forward);
-		gtk_widget_hide (assistant->back);
-		gtk_widget_hide (assistant->apply);
-		gtk_widget_hide (assistant->last);
-		gtk_widget_hide (assistant->cancel);
-		gtk_widget_show (assistant->close);
-		gtk_widget_set_sensitive (assistant->close, FALSE);
-	}
-	return FALSE;
-}
-
 void prepare_callback (GtkWidget *assistant,
 		       GtkWidget *page,
 		       gpointer data)
 {
 	gboolean complete = TRUE;
 
+	gtk_widget_hide (button_close);
+	gtk_widget_hide (button_cancel);
+
 	if (page == page_search) {
 		complete = set_page_search_complete ();
 		bluetooth_chooser_start_discovery(selector);
@@ -507,11 +437,13 @@ void prepare_callback (GtkWidget *assistant,
 
 		/* translators:
 		 * The '%s' is the device name, for example:
-		 * Connecting to 'Sony Bluetooth Headset' now...
+		 * Connecting to 'Sony Bluetooth Headset'...
 		 */
 		text = g_strdup_printf (_("Connecting to '%s'..."), target_name);
 		gtk_label_set_text (GTK_LABEL (label_connecting), text);
 		g_free (text);
+
+		gtk_widget_show (button_cancel);
 	} else {
 		gtk_spinner_stop (GTK_SPINNER (spinner_connecting));
 	}
@@ -555,6 +487,8 @@ void prepare_callback (GtkWidget *assistant,
 		} else {
 			g_assert_not_reached ();
 		}
+
+		gtk_widget_show (button_cancel);
 	}
 
 	if (page == page_finishing) {
@@ -571,6 +505,8 @@ void prepare_callback (GtkWidget *assistant,
 		text = g_strdup_printf (_("Please wait while finishing setup on device '%s'..."), target_name);
 		gtk_label_set_text (GTK_LABEL (label_finishing), text);
 		g_free (text);
+
+		gtk_widget_show (button_close);
 	} else {
 		gtk_spinner_stop (GTK_SPINNER (spinner_finishing));
 	}
@@ -615,15 +551,20 @@ void prepare_callback (GtkWidget *assistant,
 	if (page == page_failure) {
 		complete = FALSE;
 		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("restart_button"));
+		gtk_widget_show (button_close);
 	} else {
 		if (gtk_widget_get_parent (W("restart_button")) != NULL)
 			gtk_assistant_remove_action_widget (GTK_ASSISTANT (assistant), W("restart_button"));
 	}
 
-	if (page == page_ssp_setup && display_called == FALSE) {
-		complete = FALSE;
-		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("matches_button"));
-		gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
+	if (page == page_ssp_setup) {
+		if (display_called == FALSE) {
+			complete = FALSE;
+			gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("matches_button"));
+			gtk_assistant_add_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
+		} else {
+			gtk_widget_show (button_cancel);
+		}
 	} else {
 		if (gtk_widget_get_parent (W("does_not_match_button")) != NULL)
 			gtk_assistant_remove_action_widget (GTK_ASSISTANT (assistant), W("does_not_match_button"));
@@ -633,11 +574,6 @@ void prepare_callback (GtkWidget *assistant,
 
 	gtk_assistant_set_page_complete (GTK_ASSISTANT(assistant),
 					 page, complete);
-
-	/* HACK to allow hiding/showing the buttons
-	 * instead of relying on the GtkAssistant doing that
-	 * for us */
-	g_idle_add (prepare_idle_cb, NULL);
 }
 
 static gboolean
@@ -671,8 +607,8 @@ entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 	if (event->length == 0)
 		return FALSE;
 
-	if ((event->keyval >= GDK_0 && event->keyval <= GDK_9) ||
-	    (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9))
+	if ((event->keyval >= GDK_KEY_0 && event->keyval <= GDK_KEY_9) ||
+	    (event->keyval >= GDK_KEY_KP_0 && event->keyval <= GDK_KEY_KP_9))
 		return FALSE;
 
 	return TRUE;
@@ -892,6 +828,14 @@ create_wizard (void)
 
 	gtk_assistant_set_forward_page_func (assistant, page_func, NULL, NULL);
 
+	/* The 2 custom buttons */
+	button_close = W("close_button");
+	button_cancel = W("cancel_button");
+	gtk_assistant_add_action_widget (assistant, button_close);
+	gtk_assistant_add_action_widget (assistant, button_cancel);
+	gtk_widget_hide (button_close);
+	gtk_widget_hide (button_cancel);
+
 	/* Intro page */
 	combo = gtk_combo_box_new();
 
@@ -963,6 +907,7 @@ create_wizard (void)
 	radio_0000 = W("radio_0000");
 	radio_1111 = W("radio_1111");
 	radio_1234 = W("radio_1234");
+	radio_none = W("radio_none");
 	radio_custom = W("radio_custom");
 	entry_custom = W("entry_custom");
 
@@ -970,6 +915,7 @@ create_wizard (void)
 	g_object_set_data (G_OBJECT (radio_0000), "pin", "0000");
 	g_object_set_data (G_OBJECT (radio_1111), "pin", "1111");
 	g_object_set_data (G_OBJECT (radio_1234), "pin", "1234");
+	g_object_set_data (G_OBJECT (radio_none), "pin", "NULL");
 	g_object_set_data (G_OBJECT (radio_custom), "entry", entry_custom);
 
 	gtk_builder_connect_signals(builder, NULL);
@@ -982,11 +928,10 @@ create_wizard (void)
 }
 
 static void
-activated_cb (GtkApplication    *app,
-	      GVariant          *variant,
-	      gpointer           user_data)
+activate_cb (GApplication      *app,
+	     gpointer           user_data)
 {
-        gtk_window_present (GTK_WINDOW (user_data));
+	gtk_window_present_with_time (GTK_WINDOW (user_data), GDK_CURRENT_TIME);
 }
 
 static GOptionEntry options[] = {
@@ -1013,8 +958,15 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
-	app = gtk_application_new ("org.gnome.Bluetooth.wizard", &argc, &argv);
-	if (g_application_is_remote (G_APPLICATION (app))) {
+	app = gtk_application_new ("org.gnome.Bluetooth.wizard", G_APPLICATION_FLAGS_NONE);
+	if (g_application_register (G_APPLICATION (app), NULL, &error) == FALSE) {
+		g_warning ("Could not register application: %s", error->message);
+		g_error_free (error);
+		return 1;
+	}
+
+	if (g_application_get_is_remote (G_APPLICATION (app))) {
+		g_application_activate (G_APPLICATION (app));
 		gdk_notify_startup_complete ();
 		return 0;
 	}
@@ -1026,6 +978,7 @@ int main (int argc, char **argv)
 	client = bluetooth_client_new();
 
 	agent = bluetooth_agent_new();
+	g_object_add_weak_pointer (G_OBJECT (agent), (gpointer *) (&agent));
 
 	bluetooth_agent_set_pincode_func(agent, pincode_callback, NULL);
 	bluetooth_agent_set_display_func(agent, display_callback, NULL);
@@ -1042,14 +995,15 @@ int main (int argc, char **argv)
 	gtk_application_add_window (app,
 				    GTK_WINDOW (window_assistant));
 
-	g_signal_connect (app, "activated",
-			  G_CALLBACK (activated_cb), window_assistant);
+	g_signal_connect (app, "activate",
+			  G_CALLBACK (activate_cb), window_assistant);
 
-	gtk_application_run (app);
+	g_application_run (G_APPLICATION (app), argc, argv);
 
 	bluetooth_plugin_manager_cleanup ();
 
-	g_object_unref(agent);
+	if (agent != NULL)
+		g_object_unref (agent);
 
 	g_object_unref(client);
 
